"""
This package collects together tools for radio astronomical interferometry.
In addition to pure-python phasing, calibration, imaging, and
deconvolution code, this package includes interfaces to MIRIAD (a Fortran
interferometry package) and HEALPix (a package for representing spherical
data sets), and some math/fitting routines from SciPy.
All code provided is released under the GNU General Public License
(see LICENSE.txt).

Author: Aaron Parsons
"""

from __future__ import print_function, division, absolute_import

from . import phs, const, coord, deconv
from . import fit, healpix, img
from . import interp, cal, map, miriad
import scipy.optimize as optimize
from . import rfi, amp, scripting, src, _src, utils
from . import dsp
from . import pol, twodgauss #added by dfm
import ephem

try:
    from .__gitlog__ import __gitlog__
    from .__branch__ import __branch__
    from .__version__ import __version__
except ImportError:
    __gitlog__ = None
    __branch__ = None
    fh = open('VERSION', 'r')
    __version__ = fh.read()
    fh.close()



import unittest
import aipy.dsp as dsp

class TestWindows(unittest.TestCase):
    def test_kaiser2(self):
        win = dsp.gen_window(1024, 'kaiser2')
        self.assertAlmostEqual(win[0], 0.01147993)
        self.assertAlmostEqual(win[1], 0.01192681)
        self.assertAlmostEqual(win[2], 0.01238142)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import unittest, aipy.healpix as h, numpy as n

class TestHealpixBase(unittest.TestCase):
    def setUp(self):
        self.hpb = h.HealpixBase(nside=32, scheme='RING')
    def test_order(self):
        """Test HEALpix order functional attribute"""
        self.assertEqual(self.hpb.order(), 5)
    def test_nside(self):
        """Test HEALpix nside functional attribute"""
        self.assertEqual(self.hpb.nside(), 32)
    def test_npix(self):
        """Test HEALpix npix functional attribute"""
        self.assertEqual(self.hpb.npix(), 12*self.hpb.nside()**2)
    def test_scheme(self):
        """Test HEALpix scheme functional attribute"""
        self.assertEqual(self.hpb.scheme(), 'RING')
    def test_npix2nside(self):
        """Test HEALpix npix2nside functional attribute"""
        self.assertEqual(self.hpb.npix2nside(12*2**12), 2**6)
    def test_set_nside_scheme(self):
        self.hpb.set_nside_scheme(64,'NEST')
        self.assertEqual(self.hpb.nside(), 64)
        self.assertEqual(self.hpb.scheme(), 'NEST')
        self.hpb.set_nside_scheme(32,'RING')
        self.assertEqual(self.hpb.nside(), 32)
        self.assertEqual(self.hpb.scheme(), 'RING')
    def test_nest_ring_conv(self):
        ipx = n.array([0])
        px = self.hpb.nest_ring_conv(ipx, 'NEST')
        self.assertEqual(px, 1023)
        self.hpb.set_nside_scheme(32, 'NEST')
        ipx = n.array([0])
        px = self.hpb.nest_ring_conv(ipx, 'RING')
        self.assertEqual(px, 5968)
    def test_ang2px(self):
        th,ph = n.linspace(0,n.pi,10), n.linspace(-n.pi,n.pi,10)
        px = self.hpb.crd2px(th,ph)
        self.assertEqual(len(px), len(th))
        n.testing.assert_allclose(px, n.array([    2,   398,  1375,  3114,  5049,  7239,  9173, 10912, 11889, 12286]))
    def test_vec2px(self):
        x,y = n.linspace(-.5,.5,10), n.linspace(-.5,.5,10)
        z = 1 - n.sqrt(x**2 + y**2)
        px = self.hpb.crd2px(x,y,z)
        self.assertEqual(len(px), len(x))
        n.testing.assert_allclose(px, n.array([3728, 2192, 1069,  247,   19,   13,  225, 1023, 2128, 3664]))
    def test_ang2px_interp(self):
        th,ph = n.linspace(0,n.pi,3), n.linspace(-n.pi,n.pi,3)
        px,wgt = self.hpb.crd2px(th,ph,interpolate=True)
        self.assertEqual(len(px), len(th))
        self.assertEqual(wgt.shape, (len(th),4))
        n.testing.assert_allclose(px, n.array(
            # The ordering of pixels can be healpix version dependent
              #[[    2,     3,     1,     0],
              # [ 6208,  6080,  6207,  5952],
              # [12286, 12285, 12287, 12284]]))
              [[    3,     0,     1,     2],
               [ 6207,  6080,  6208,  6209],
               [12285, 12286, 12287, 12284]]))
        n.testing.assert_allclose(wgt, n.array(
            # The ordering of pixels can be healpix version dependent
              #[[ 0.25,  0.25,  0.25,  0.25],
              # [ 0.25,  0.25,  0.25,  0.25],
              # [ 0.25,  0.25,  0.25,  0.25]]))
              [[ 0.25,  0.25,  0.25,  0.25],
               [ 0.5 ,  0.5 ,  0.  ,  0.  ],
               [ 0.25,  0.25,  0.25,  0.25]]))
    def test_vec2px_interp(self):
        x,y = n.linspace(-.5,.5,3), n.linspace(-.5,.5,3)
        z = 1 - n.sqrt(x**2 + y**2)
        px,wgt = self.hpb.crd2px(x,y,z,interpolate=True)
        self.assertEqual(len(px), len(x))
        self.assertEqual(wgt.shape, (len(x),4))
        n.testing.assert_allclose(px, n.array(
            # The ordering of pixels can be healpix version dependent
              #[[3984, 3856, 3855, 3728],
              # [   0,    1,    3,    2],
              # [3920, 3792, 3791, 3664]]))
              [[3728, 3729, 3855, 3856],
               [   1,    2,    3,    0],
               [3664, 3665, 3791, 3792]]))
        n.testing.assert_allclose(wgt, n.array(
            # The ordering of pixels can be healpix version dependent
              #[[ 0.099602,  0.215996,  0.215996,  0.468407],
              # [ 0.25    ,  0.25    ,  0.25    ,  0.25    ],
              # [ 0.099602,  0.215996,  0.215996,  0.468407]]), rtol=1e-5)
              [[ 0.367711,  0.      ,  0.316145,  0.316145],
               [ 0.25    ,  0.25    ,  0.25    ,  0.25    ],
               [ 0.367711,  0.      ,  0.316145,  0.316145]]), rtol=1e-5)

class TestHealpixMap(unittest.TestCase):
    def setUp(self):
        self.hpm = h.HealpixMap(32, 'RING')
    def test_set(self):
        self.hpm[0] = 1
        self.assertEqual(self.hpm[0], 1)
        self.hpm[0,0] = 1
        self.assertEqual(self.hpm[0,0], 1)
        self.hpm[0,1,0] = 1
        self.assertEqual(self.hpm[0,1,0], 1)
    def test_set_px(self):
        c = n.arange(15)
        self.assertEqual(len(self.hpm[c]), len(c))
        self.hpm[c] += c
        n.testing.assert_allclose(self.hpm[c], c)
        self.hpm[c] += c
        n.testing.assert_allclose(self.hpm[c], 2*c)
        self.hpm[c] = c
        n.testing.assert_allclose(self.hpm[c], c)
    def test_set_ang(self):
        th,ph = n.linspace(0,n.pi,10), n.linspace(-n.pi,n.pi,10)
        self.assertEqual(len(self.hpm[th,ph]), len(th))
        self.hpm[th,ph] += th
        n.testing.assert_allclose(self.hpm[th,ph], th)
        self.hpm[th,ph] += th
        n.testing.assert_allclose(self.hpm[th,ph], 2*th)
        self.hpm[th,ph] = th
        n.testing.assert_allclose(self.hpm[th,ph], th)
    def test_set_vec(self):
        x,y = n.linspace(-.5,.5,10), n.linspace(-.5,.5,10)
        z = 1 - n.sqrt(x**2 + y**2)
        self.assertEqual(len(self.hpm[x,y,z]), len(x))
        self.hpm[x,y,z] += x
        n.testing.assert_allclose(self.hpm[x,y,z], x)
        self.hpm[x,y,z] += x
        n.testing.assert_allclose(self.hpm[x,y,z], 2*x)
        self.hpm[x,y,z] = x
        n.testing.assert_allclose(self.hpm[x,y,z], x)
        

if False:
  class TestMemLeaks(unittest.TestCase):
    def setUp(self):
        self.hpb = h.HealpixBase(nside=256)
    def test_create(self):
        while True: hpb = h.HealpixBase(nside=256)
    def test_crd2pix(self):
        one = 0.99
        x = n.arange(-one, one, .01, dtype=n.double)
        y = n.zeros_like(x)
        z = n.sqrt(1 - x**2 - y**2)
        while True: crds = self.hpb.crd2px(x, y, z)

class TestSuite(unittest.TestSuite):
    """A unittest.TestSuite class which contains all of the aipy._healpix unit tests."""

    def __init__(self):
        unittest.TestSuite.__init__(self)

        loader = unittest.TestLoader()
        self.addTests(loader.loadTestsFromTestCase(TestHealpix))
        #self.addTests(loader.loadTestsFromTestCase(TestMemLeaks))

if __name__ == '__main__':
    unittest.main()

#hp = aipy._healpix.HealpixBase(2**6, 'NEST')
#print hp.order(), hp.nside(), hp.npix(), hp.scheme()
#px = numpy.arange(hp.npix())
#px = hp.nest_ring_conv(px, 'RING')
#hp.set_nside_scheme(hp.nside(), 'RING')
#px = numpy.arange(10)
#x,y,z = hp.px2crd(px, ncrd=3)
#th,phi = hp.px2crd(px, ncrd=2)
#assert(numpy.all(px == hp.crd2px(x,y,z, interpolate=0)))
#px2,wgt = hp.crd2px(x,y,z, interpolate=1)
#z[-1] = numpy.Inf
#try: hp.crd2px(x,y,z[:-1])
#except(RuntimeError): pass
#assert(numpy.all(px == hp.crd2px(th, phi, interpolate=0)))
#px2,wgt = hp.crd2px(th, phi, interpolate=1)



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import unittest
try:
    import aipy.healpix as a
except ImportError:
    import aipy_src.healpix as a
import numpy as np

class TestAlm(unittest.TestCase):
    def test_init(self):
        """Test initializing a aipy._alm object"""
        alm = a.Alm(10, 5)
        self.assertEqual(alm.lmax(), 10)
        self.assertEqual(alm.mmax(), 5)
        self.assertRaises(Exception, a.Alm, 5, 10)
    def test_get_data(self):
        alm = a.Alm(1,1)
        data = alm.get_data()
        self.assertEqual(data.size, 3)
    def test_set_data(self):
        alm = a.Alm(1,1)
        data = np.array([1+1j, 2+2j, 3+3j], dtype=np.complex128)
        alm.set_data(data)
        np.testing.assert_equal(alm.get_data(), data)
    def test_set_to_zero(self):
        alm = a.Alm(3,3)
        np.testing.assert_equal(alm.get_data(), 0)
    def test_get_set(self):
        alm = a.Alm(1,1)
        data = np.array([1+1j, 2+2j, 3+3j], dtype=np.complex128)
        alm.set_data(data)
        self.assertEqual(alm[0,0], data[0])
        self.assertEqual(alm[1,0], data[1])
        self.assertEqual(alm[1,1], data[2])
        alm[1,1] = 4+4j
        self.assertEqual(alm[1,1], 4+4j)
    def test_to_map(self):
        alm = a.Alm(1,1)
        alm[0,0] = 1
        d = alm.to_map(32, 'RING')
        np.testing.assert_almost_equal(d, np.ones(12*32**2, dtype=np.float) * 0.2820948)
    def test_from_map(self):
        alm = a.Alm(1,1)
        d = np.ones(12*32**2, dtype=np.float32)
        alm.from_map(d, 3)
        self.assertAlmostEqual(alm[0,0], 3.5449077018110313)
        self.assertAlmostEqual(alm[1,0], 0)
        self.assertAlmostEqual(alm[1,1], 0)

if False:
  class TestMemLeaks(unittest.TestCase):
    def setUp(self):
        self.alm = a.Alm(10,10)
    def test_alm_create(self):
        while True: alm = a.Alm(20,20)
    def test_alm_to_map(self):
        while True: d = self.alm.to_map(256, 'RING')
    def test_alm_from_map(self):
        d = np.zeros(12*256**2, dtype=n.float)
        while True: self.alm.from_map(d, 2)
    def test_alm_get_data(self):
        while True: c = self.alm.get_data()
    def test_alm_set_data(self):
        c = self.alm.get_data()
        while True: self.alm.set_data(c)

class TestSuite(unittest.TestSuite):
    """A unittest.TestSuite class which contains all of the aipy._alm unit tests."""

    def __init__(self):
        unittest.TestSuite.__init__(self)

        loader = unittest.TestLoader()
        self.addTests(loader.loadTestsFromTestCase(TestAlm))
        #self.addTests(loader.loadTestsFromTestCase(TestMemLeaks))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import unittest, numpy as n
import aipy.twodgauss as g

class TestTwodgauss(unittest.TestCase):
    def test_moments(self):
        """Test twodgauss.moments"""

        a0 = 2.0
        x0, y0 = 5.0, 7.0
        wx, wy = 1.0, 1.0
        x, y = n.arange(11, dtype=n.float64), n.arange(11, dtype=n.float64)
        x, y = n.meshgrid(x, y)
        data = a0*n.exp(-((x-x0)/wx)**2 / 2.0 - ((y-y0)/wy)**2 / 2.0)
        params = g.moments(data)
        self.assertAlmostEqual(a0, params[1], 3)
        self.assertAlmostEqual(x0, params[2], 3)
        self.assertAlmostEqual(y0, params[3], 3)
        #self.assertAlmostEqual(wx, params[4], 3)	# Why not?
        #self.assertAlmostEqual(wy, params[5], 3)	# Why not?

    def test_twodgaussian(self):
        """Test twodgauss.twodgaussian"""

        a0 = 2.0
        x0, y0 = 5.0, 7.0
        wx, wy = 1.0, 1.0
        x, y = n.arange(11, dtype=n.float64), n.arange(11, dtype=n.float64)
        x, y = n.meshgrid(x, y)
        data0 = a0*n.exp(-((x-x0)/wx)**2 / 2.0 - ((y-y0)/wy)**2 / 2.0)
        data1 = g.twodgaussian([0.0, a0, x0, y0, wx, wy], shape=y.shape)
        for i in range(data0.shape[0]):
            for j in range(data0.shape[1]):
                self.assertAlmostEqual(data0[i,j], data1[i,j], 6)

class TestSuite(unittest.TestSuite):
    """A unittest.TestSuite class which contains all of the aipy.phs unit tests."""

    def __init__(self):
        unittest.TestSuite.__init__(self)

        loader = unittest.TestLoader()
        self.addTests(loader.loadTestsFromTestCase(TestTwodgauss))

if __name__ == '__main__':
    unittest.main()
